@@ -82,7 +82,7 @@ typedef struct {
 	int color_idx;
 	int boldness;       /* 100->1000 dec, normal = 0x190, bold = 0x2bc */
 	GOFontScript script;
-	PangoUnderline underline;
+	MsBiffFontUnderline underline;
 	char *fontname;
 	PangoAttrList *attrs;
 	GOFont const  *go_font;
@@ -100,7 +100,7 @@ typedef struct {
 
 struct _GnmXLImporter {
 	MSContainer	  container;
-	IOContext	 *context;
+	GOIOContext	 *context;
 	WorkbookView	 *wbv;
 	Workbook         *wb;
 	MsBiffVersion	  ver;
@@ -174,6 +174,4 @@ void xls_read_SXIVD	 (BiffQuery *q, ExcelReadSheet *esheet);
 void excel_read_cleanup (void);
 void excel_read_init (void);
 
-unsigned xl_pattern_to_line_type (guint16 pattern);
-
 #endif /* GNM_MS_EXCEL_READ_H */
