@@ -35,6 +35,7 @@
 #include <value.h>
 #include <cell.h>
 #include <sheet.h>
+#include <sheet-view.h>
 #include <expr.h>
 #include <wbc-gtk.h>
 #include <workbook.h>
@@ -628,12 +629,13 @@ run_solver (SolverState *state, GnmSolverParameters *param)
 	GnmSolverResult *res = NULL;
 	int y;
 
-	sol = param->options.algorithm
-		? gnm_solver_factory_create (param->options.algorithm, param)
-		: NULL;
+	sol = gnm_solver_factory_functional (param->options.algorithm,
+					     state->wbcg)
+	    ? gnm_solver_factory_create (param->options.algorithm, param)
+	    : NULL;
 	if (!sol) {
 		go_gtk_notice_dialog (top, GTK_MESSAGE_ERROR,
-				      _("No suitable solver available."));
+				      _("The chosen solver is not functional."));
 		goto fail;
 	}
 
@@ -907,7 +909,6 @@ dialog_init (SolverState *state)
 	GnmCell *target_cell;
 	GnmValue const *input;
 	int i;
-	GnmRange const *first;
 
 	param = state->sheet->solver_parameters;
 
@@ -1128,6 +1129,15 @@ dialog_init (SolverState *state)
 	if (target_cell)
 		gnm_expr_entry_load_from_text (state->target_entry,
 					       cell_name (target_cell));
+	else {
+		SheetView *sv = wb_control_cur_sheet_view 
+			(WORKBOOK_CONTROL (state->wbcg));
+		if (sv) {
+			GnmRange first = {sv->edit_pos, sv->edit_pos};
+			gnm_expr_entry_load_from_range (state->target_entry,
+							state->sheet, &first);
+		}
+	}
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
 		glade_xml_get_widget(state->gui, "max_button")),
@@ -1148,14 +1158,6 @@ dialog_init (SolverState *state)
 			    param->options.scenario_name);
 
 /* Done */
-	first = selection_first_range 
-		(wb_control_cur_sheet_view 
-		 (WORKBOOK_CONTROL (state->wbcg)), NULL, NULL);
-	if (first != NULL) {
-		gnm_expr_entry_load_from_range (state->target_entry,
-						state->sheet, first);
-	}
-	
 	gnm_expr_entry_grab_focus (state->target_entry, FALSE);
 	wbcg_set_entry (state->wbcg, state->target_entry);
 
@@ -1189,19 +1191,29 @@ dialog_solver (WBCGtk *wbcg, Sheet *sheet)
 {
         SolverState *state;
 	GnmSolverParameters *old_params = sheet->solver_parameters;
+	gboolean got_it;
+	int pass;
 
 	/* Only pop up one copy per workbook */
 	if (gnumeric_dialog_raise_if_exists (wbcg, SOLVER_KEY))
 		return;
 
-	/* First time around, pick a functional algorithm.  */
-	if (!gnm_solver_factory_functional (old_params->options.algorithm)) {
+	/*
+	 * First time around, pick a functional algorithm preferably one we
+	 * can determine is functional without asking the user anything.
+	 */
+	got_it = gnm_solver_factory_functional (old_params->options.algorithm,
+						NULL);
+	for (pass = 1; !got_it && pass <= 2; pass++) {
 		GSList *l;
+		WBCGtk *wbcg2 = pass == 2 ? wbcg : NULL;
+
 		for (l = gnm_solver_db_get (); l; l = l->next) {
 			GnmSolverFactory *factory = l->data;
 			if (old_params->options.model_type != factory->type)
 				continue;
-			if (gnm_solver_factory_functional (factory)) {
+			if (gnm_solver_factory_functional (factory, wbcg2)) {
+				got_it = TRUE;
 				gnm_solver_param_set_algorithm (old_params,
 								factory);
 				break;
