@@ -188,7 +188,8 @@ encodings_changed_cb (GOCharmapSel *cs, char const *new_charmap,
 			   "please select another encoding."),
 			 name ? name : new_charmap);
 		go_gtk_notice_dialog (GTK_WINDOW (pagedata->dialog),
-				 GTK_MESSAGE_ERROR, msg);
+				      GTK_MESSAGE_ERROR,
+				      "%s", msg);
 		g_free (msg);
 
 		go_charmap_sel_set_encoding (pagedata->main.charmap_selector,
@@ -417,10 +418,10 @@ stf_dialog_main_page_init (GladeXML *gui, StfDialogData *pagedata)
 
 	/* Connect signals */
 	g_signal_connect (G_OBJECT (pagedata->main.main_startrow),
-		"value-changed",
+		"changed",
 		G_CALLBACK (main_page_startrow_changed), pagedata);
 	g_signal_connect (G_OBJECT (pagedata->main.main_stoprow),
-		"value-changed",
+		"changed",
 		G_CALLBACK (main_page_stoprow_changed), pagedata);
 	g_signal_connect (G_OBJECT (pagedata->main.main_separated),
 		"toggled",
