@@ -196,9 +196,11 @@ text_content_received (GtkClipboard *clipboard,  GtkSelectionData *sel,
 			    sel->length,
 			    name);
 		g_free (name);
-		gsf_mem_dump (sel->data, MAX (sel->length, maxlen));
-		if (sel->length > maxlen)
-			g_printerr ("...\n");
+		if (sel->length > 0) {
+			gsf_mem_dump (sel->data, MIN (sel->length, maxlen));
+			if (sel->length > maxlen)
+				g_printerr ("...\n");
+		}
 	}
 
 	/* Nothing on clipboard? */
@@ -347,9 +349,11 @@ image_content_received (GtkClipboard *clipboard, GtkSelectionData *sel,
 			    sel->length,
 			    name);
 		g_free (name);
-		gsf_mem_dump (sel->data, MAX (sel->length, maxlen));
-		if (sel->length > maxlen)
-			g_printerr ("...\n");
+		if (sel->length > 0) {
+			gsf_mem_dump (sel->data, MIN (sel->length, maxlen));
+			if (sel->length > maxlen)
+				g_printerr ("...\n");
+		}
 	}
 
 	if (sel->length > 0) {
@@ -366,6 +370,83 @@ image_content_received (GtkClipboard *clipboard, GtkSelectionData *sel,
 	}
 }
 
+static void
+parse_ms_headers (const char *data, size_t length, size_t *start, size_t *end)
+{
+	GHashTable *headers = g_hash_table_new_full
+		(g_str_hash, g_str_equal, g_free, g_free);
+	size_t limit = length;
+	size_t i = 0;
+	char *key = NULL;
+	char *value = NULL;
+	long sf, ef;
+	const char *v;
+
+	while (i < limit && data[i] != '<') {
+		size_t j, k;
+
+		for (j = i; j < limit; j++) {
+			if (data[j] == ':') {
+				key = g_strndup (data + i, j - i);
+				break;
+			}
+			if (g_ascii_isspace (data[j]))
+				goto bad;
+		}
+		if (j >= limit)
+			goto bad;
+		j++;
+
+		for (k = j; k < limit; k++) {
+			if (data[k] == '\n' || data[k] == '\r') {
+				value = g_strndup (data + j, k - j);
+				break;
+			}
+		}
+		if (k >= limit)
+			goto bad;
+		while (g_ascii_isspace (data[k]))
+			k++;
+
+		i = k;
+
+		if (debug_clipboard ())
+			g_printerr ("MS HTML Header [%s] => [%s]\n", key, value);
+
+		if (strcmp (key, "StartHTML") == 0) {
+			long l = strtol (value, NULL, 10);
+			limit = MIN (limit, MAX (0, l));
+		}
+
+		g_hash_table_replace (headers, key, value);
+		key = value = NULL;
+	}
+
+	v = g_hash_table_lookup (headers, "StartFragment");
+	sf = v ? strtol (v, NULL, 10) : -1;
+	if (sf < (long)limit)
+		goto bad;
+
+	v = g_hash_table_lookup (headers, "EndFragment");
+	ef = v ? strtol (v, NULL, 10) : -1;
+	if (ef < sf || ef > (long)length)
+		goto bad;
+
+	*start = sf;
+	*end = ef;
+	goto out;
+
+ bad:
+	g_free (key);
+	g_free (value);
+	*start = 0;
+	*end = length;
+
+ out:
+	g_hash_table_destroy (headers);
+}
+
+
 static void
 table_content_received (GtkClipboard *clipboard, GtkSelectionData *sel,
 			gpointer closure)
@@ -383,9 +464,11 @@ table_content_received (GtkClipboard *clipboard, GtkSelectionData *sel,
 			    sel->length,
 			    name);
 		g_free (name);
-		gsf_mem_dump (sel->data, MAX (sel->length, maxlen));
-		if (sel->length > maxlen)
-			g_printerr ("...\n");
+		if (sel->length > 0) {
+			gsf_mem_dump (sel->data, MIN (sel->length, maxlen));
+			if (sel->length > maxlen)
+				g_printerr ("...\n");
+		}
 	}
 
 	/* Nothing on clipboard? */
@@ -409,9 +492,18 @@ table_content_received (GtkClipboard *clipboard, GtkSelectionData *sel,
 						 sel->length);
 	} else if (sel->target == gdk_atom_intern (HTML_ATOM_NAME_UNIX, FALSE) ||
 		   sel->target == gdk_atom_intern (HTML_ATOM_NAME_WINDOWS, FALSE)) {
+		size_t length = sel->length;
+		size_t start = 0, end = length;
+
+		if (sel->target == gdk_atom_intern (HTML_ATOM_NAME_WINDOWS, FALSE)) {
+			/* See bug 143084 */
+			parse_ms_headers (sel->data, length, &start, &end);
+		}
+
 		content = table_cellregion_read (wbc, "Gnumeric_html:html",
-						 pt, sel->data,
-						 sel->length);
+						 pt,
+						 sel->data + start,
+						 end - start);
 	} else if ((sel->target == gdk_atom_intern ( BIFF8_ATOM_NAME, FALSE)) ||
 		   (sel->target == gdk_atom_intern ( BIFF8_ATOM_NAME_CITRIX, FALSE)) ||
 		   (sel->target == gdk_atom_intern ( BIFF5_ATOM_NAME, FALSE)) ||
@@ -939,7 +1031,7 @@ target_list_to_entries (GtkTargetList *target_list, int *n_entries)
 /* We include bmp in the whitelist because that's the only image format
  * we share with OOo over clipboard (!) */
 static void
-set_clipman_targets (GdkDisplay *disp, GtkTargetEntry *targets, int n_targets)
+set_clipman_targets (GdkDisplay *disp, GtkTargetEntry *targets, guint n_targets)
 {
 	static GtkTargetEntry clipman_whitelist[] = {
 		{ (char *) GNUMERIC_ATOM_NAME, 0, GNUMERIC_ATOM_INFO },
@@ -953,14 +1045,14 @@ set_clipman_targets (GdkDisplay *disp, GtkTargetEntry *targets, int n_targets)
 		{ (char *)"image/jpeg", 0, 0 },
 		{ (char *)"image/bmp", 0, 0 },
 	};
-	int n_whitelist = G_N_ELEMENTS (clipman_whitelist);
+	guint n_whitelist = G_N_ELEMENTS (clipman_whitelist);
 	int n_allowed;
 	GtkTargetList *tl = gtk_target_list_new (NULL, 0);
 	GtkTargetEntry *t, *wt, *t_allowed;
 
-	for (t = targets; t - targets < n_targets; t++) {
+	for (t = targets; t < targets + n_targets; t++) {
 		for (wt = clipman_whitelist;
-		     wt - clipman_whitelist < n_whitelist; wt++) {
+		     wt < clipman_whitelist + n_whitelist; wt++) {
 			if (strcmp(t->target, wt->target) == 0) {
 				gtk_target_list_add
 					(tl, gdk_atom_intern (t->target, FALSE),
