@@ -119,7 +119,7 @@ struct _GnmExprEntry {
 		GnmFunc         *fd;
 		gint             args;
 		gboolean         had_stuff;
-		gulong           handlerid;
+		gulong            handlerid;
 		gboolean         enabled;
 		gboolean         completion_se_valid;
 		gchar           *completion;
@@ -1122,12 +1122,196 @@ cb_gee_key_press_event (GtkEntry	*entry,
 	int state = gnumeric_filter_modifiers (event->state);
 
 	switch (event->keyval) {
-	case GDK_Up:	case GDK_KP_Up:
-	case GDK_Down:	case GDK_KP_Down:
-		if (gee->is_cell_renderer)
-			return FALSE;
-		/* Ignore these keys */
+	case GDK_Up:	
+	case GDK_KP_Up:
+		/* Tab is only applicable for the main entry */
+		 if (gee->is_cell_renderer || !wbcg_is_editing (wbcg))
+			break;
+		{
+		SheetView *sv;
+		WBCEditResult result;
+
+		if(blockArrows){
+			gtk_editable_set_position(wbcg_get_entry (wbcg), 0);
+			return TRUE;
+		}
+		else{
+
+			if (is_enter && (state & GDK_CONTROL_MASK))
+				result = (state & GDK_SHIFT_MASK) ? WBC_EDIT_ACCEPT_ARRAY : WBC_EDIT_ACCEPT_RANGE;
+			else
+				result = WBC_EDIT_ACCEPT;
+
+			/* Be careful to restore the editing sheet if we are editing */
+			sv = sheet_get_view (wbcg->editing_sheet,
+				wb_control_view (WORKBOOK_CONTROL (wbcg)));
+
+			/* move the edit pos for normal entry */
+			if (wbcg_edit_finish (wbcg, result, NULL) && result == WBC_EDIT_ACCEPT) {
+				GODirection dir = gnm_conf_get_core_gui_editing_enter_moves_dir ();
+				if (!is_enter || dir != GO_DIRECTION_NONE) {
+					gboolean forward = FALSE;
+					gboolean horizontal = FALSE;
+					if (is_enter) {
+						horizontal = go_direction_is_horizontal (dir);
+						forward = go_direction_is_forward (dir);
+					}
+	
+					if (event->state & GDK_SHIFT_MASK)
+						forward = !forward;
+
+					sv_selection_walk_step2(sv, forward, horizontal);
+	
+					/* invalidate, in case Enter direction changes */
+					if (is_enter)
+						sv->first_tab_col = -1;
+					sv_update (sv);
+				}
+			}
+			return TRUE;
+		}
+	}
+	case GDK_Down:	
+	case GDK_KP_Down:
+		//if(blockArrows)
+		//	break;
+		/* Tab is only applicable for the main entry */
+		 if (gee->is_cell_renderer || !wbcg_is_editing (wbcg))
+			break;
+		{
+		SheetView *sv;
+		WBCEditResult result;
+
+		if(blockArrows){
+			gtk_editable_set_position(wbcg_get_entry (wbcg), -1);
+			return TRUE;
+		}
+		else{
+
+			if (is_enter && (state & GDK_CONTROL_MASK))
+				result = (state & GDK_SHIFT_MASK) ? WBC_EDIT_ACCEPT_ARRAY : WBC_EDIT_ACCEPT_RANGE;
+			else
+				result = WBC_EDIT_ACCEPT;
+
+			/* Be careful to restore the editing sheet if we are editing */
+			sv = sheet_get_view (wbcg->editing_sheet,
+				wb_control_view (WORKBOOK_CONTROL (wbcg)));
+	
+			/* move the edit pos for normal entry */
+			if (wbcg_edit_finish (wbcg, result, NULL) && result == WBC_EDIT_ACCEPT) {
+				GODirection dir = gnm_conf_get_core_gui_editing_enter_moves_dir ();
+				if (!is_enter || dir != GO_DIRECTION_NONE) {
+					gboolean forward = TRUE;
+					gboolean horizontal = FALSE;
+					if (is_enter) {
+						horizontal = go_direction_is_horizontal (dir);
+						forward = go_direction_is_forward (dir);
+					}
+	
+					if (event->state & GDK_SHIFT_MASK)
+						forward = !forward;
+	
+					sv_selection_walk_step2(sv, forward, horizontal);
+	
+					/* invalidate, in case Enter direction changes */
+					if (is_enter)
+						sv->first_tab_col = -1;
+					sv_update (sv);
+				}
+			}
+			return TRUE;
+		}
+	}
+
+	case GDK_Left:	
+	case GDK_KP_Left:
+		if(blockArrows)
+			break;
+		/* Tab is only applicable for the main entry */
+		 if (gee->is_cell_renderer || !wbcg_is_editing (wbcg))
+			break;
+		{
+		SheetView *sv;
+		WBCEditResult result;
+
+		if (is_enter && (state & GDK_CONTROL_MASK))
+			result = (state & GDK_SHIFT_MASK) ? WBC_EDIT_ACCEPT_ARRAY : WBC_EDIT_ACCEPT_RANGE;
+		else
+			result = WBC_EDIT_ACCEPT;
+
+		/* Be careful to restore the editing sheet if we are editing */
+		sv = sheet_get_view (wbcg->editing_sheet,
+			wb_control_view (WORKBOOK_CONTROL (wbcg)));
+
+		/* move the edit pos for normal entry */
+		if (wbcg_edit_finish (wbcg, result, NULL) && result == WBC_EDIT_ACCEPT) {
+			GODirection dir = gnm_conf_get_core_gui_editing_enter_moves_dir ();
+			if (!is_enter || dir != GO_DIRECTION_NONE) {
+				gboolean forward = FALSE;
+				gboolean horizontal = TRUE;
+				if (is_enter) {
+					horizontal = go_direction_is_horizontal (dir);
+					forward = go_direction_is_forward (dir);
+				}
+
+				if (event->state & GDK_SHIFT_MASK)
+					forward = !forward;
+
+				sv_selection_walk_step2(sv, forward, horizontal);
+
+				/* invalidate, in case Enter direction changes */
+				if (is_enter)
+					sv->first_tab_col = -1;
+				sv_update (sv);
+			}
+		}
 		return TRUE;
+	}
+
+	case GDK_Right:	
+	case GDK_KP_Right:
+		if(blockArrows)
+			break;
+		/* Tab is only applicable for the main entry */
+		 if (gee->is_cell_renderer || !wbcg_is_editing (wbcg))
+			break;
+		{
+		SheetView *sv;
+		WBCEditResult result;
+
+		if (is_enter && (state & GDK_CONTROL_MASK))
+			result = (state & GDK_SHIFT_MASK) ? WBC_EDIT_ACCEPT_ARRAY : WBC_EDIT_ACCEPT_RANGE;
+		else
+			result = WBC_EDIT_ACCEPT;
+
+		/* Be careful to restore the editing sheet if we are editing */
+		sv = sheet_get_view (wbcg->editing_sheet,
+			wb_control_view (WORKBOOK_CONTROL (wbcg)));
+
+		/* move the edit pos for normal entry */
+		if (wbcg_edit_finish (wbcg, result, NULL) && result == WBC_EDIT_ACCEPT) {
+			GODirection dir = gnm_conf_get_core_gui_editing_enter_moves_dir ();
+			if (!is_enter || dir != GO_DIRECTION_NONE) {
+				gboolean forward = TRUE;
+				gboolean horizontal = TRUE;
+				if (is_enter) {
+					horizontal = go_direction_is_horizontal (dir);
+					forward = go_direction_is_forward (dir);
+				}
+
+				if (event->state & GDK_SHIFT_MASK)
+					forward = !forward;
+
+				sv_selection_walk_step2(sv, forward, horizontal);
+
+				/* invalidate, in case Enter direction changes */
+				if (is_enter)
+					sv->first_tab_col = -1;
+				sv_update (sv);
+			}
+		}
+		return TRUE;
+	}
 		/* GDK_F2 starts editing */
 		/* GDK_F3 opens the paste names dialog */
 	case GDK_F4: {
@@ -1206,12 +1390,12 @@ cb_gee_key_press_event (GtkEntry	*entry,
 		if (gee->is_cell_renderer)
 			return FALSE;
 		/* Is this the right way to append a newline ?? */
-		if (state == GDK_MOD1_MASK) {
-			gint pos = gtk_editable_get_position (GTK_EDITABLE (entry));
-			gtk_editable_insert_text (GTK_EDITABLE (entry), "\n", 1, &pos);
-			gtk_editable_set_position (GTK_EDITABLE (entry), pos);
-			return TRUE;
-		}
+		//if (state == GDK_MOD1_MASK) {
+		//	gint pos = gtk_editable_get_position (GTK_EDITABLE (entry));
+		//	gtk_editable_insert_text (GTK_EDITABLE (entry), "\n", 1, &pos);
+		//	gtk_editable_set_position (GTK_EDITABLE (entry), pos);
+		//	return TRUE;
+		//}
 
 		/* Ctrl-enter is only applicable for the main entry */
 		if (!wbcg_is_editing (wbcg))
@@ -1244,7 +1428,7 @@ cb_gee_key_press_event (GtkEntry	*entry,
 			GODirection dir = gnm_conf_get_core_gui_editing_enter_moves_dir ();
 			if (!is_enter || dir != GO_DIRECTION_NONE) {
 				gboolean forward = TRUE;
-				gboolean horizontal = TRUE;
+				gboolean horizontal = FALSE;
 				if (is_enter) {
 					horizontal = go_direction_is_horizontal (dir);
 					forward = go_direction_is_forward (dir);
@@ -1253,12 +1437,12 @@ cb_gee_key_press_event (GtkEntry	*entry,
 				if (event->state & GDK_SHIFT_MASK)
 					forward = !forward;
 
-				sv_selection_walk_step (sv, forward, horizontal);
+				//sv_selection_walk_step2(sv, forward, horizontal);
 
 				/* invalidate, in case Enter direction changes */
 				if (is_enter)
 					sv->first_tab_col = -1;
-				sv_update (sv);
+				//sv_update (sv);
 			}
 		}
 		return TRUE;
@@ -1380,9 +1564,9 @@ gee_init (GnmExprEntry *gee)
 
 	gee->entry = GTK_ENTRY (gtk_entry_new ());
 
-#ifdef GNM_USE_HILDON
-	g_object_set (G_OBJECT (gee->entry), "autocap", FALSE, NULL);
-#endif
+//#ifdef GNM_USE_HILDON
+//	g_object_set (G_OBJECT (gee->entry), "autocap", FALSE, NULL);
+//#endif
 
 	/* Disable selecting the entire content when the widget gets focus */
 	g_object_set (gtk_widget_get_settings (GTK_WIDGET (gee->entry)),
